#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>

#include "config.h"
#include "imapfilter.h"
#include "file.h"
#include "connect.h"
#include "imap.h"
#include "log.h"


extern int sock;
extern account_t *accounts;

unsigned int options;		/* Program options. */
char logfile[PATH_MAX];		/* Log file. */


/*
 * In the beginning there was main()...
 */
int main(int argc, char **argv)
{
    int c;
    char *cfg = NULL;		/* Configuration file. */
    account_t *ca;

    options = OPT_DETAILS_NORMAL;
    logfile[0] = 0;

    while ((c = getopt(argc, argv, "c:hl:qtv")) != -1) {
	switch (c) {
	case 'c':
	    cfg = optarg;
	    break;
	case 'h':
	    usage();
	    exit(1);
	    break;
	case 'l':
	    strncpy(logfile, optarg, PATH_MAX);
	    break;
	case 'q':
	    options &= OPT_DETAILS_CLEAR;
	    options |= OPT_DETAILS_QUITE;
	    break;
	case 't':
	    options |= OPT_TEST_MODE;
	    options |= OPT_TEST_MODE_STICKY;	/* Override config
						   file's TEST_MODE. */
	    break;
	case 'v':
	    options &= OPT_DETAILS_CLEAR;
	    options |= OPT_DETAILS_VERBOSE;
	    break;
	default:
	    usage();
	    exit(1);
	    break;
	}
    }

    if (read_config(cfg) || open_logfile())
	exit(2);

    for (ca = accounts; ca; ca = ca->next) {


	if (init_connection(ca)) {
	    close_logfile();
	    exit(3);
	}

	clear_stream();

#ifdef DEBUG
	imap_noop();
#endif

	imap_login(ca);
	imap_select();
	imap_status();

	apply_filters();

	imap_logout();

	close_connection();
    }

    close_logfile();

    exit(0);
}


/*
 * Searches and deletes any messages (if found and not in test mode) that
 * match the defined filters.
 */
void apply_filters(void)
{
    char res[SEARCH_BUF];

    if (!imap_search(res) && res[0]) {
	del_messages(res);
	if (!(options & OPT_TEST_MODE))
	    imap_expunge();
    }
}


/*
 * Prints a very brief usage message.
 */
void usage(void)
{
    fprintf(stderr,
	    "usage: imapfilter [-hqtv] [-c configfile] [-l logfile]\n");
}
